#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# The person who does the installation gets a lot of information that others
# don't need. For example, database passwords, and whatnot. However, subsequent
# users need to be able to access the same back-end and interact with it.
#
# To allow this to happen, you can 'join' a team and receive the credentials
# necessary to access the environment's resources. In the future we may want
# to use this to create more fine-grain access to resources, but for now,
# it just handles the situation where someone can get a subset of the credentials
# onto their machine so they can co-develop.
#
# By leaving a team, the team credentials for that profile will be removed from the
# local directory.
#
# At this point, this is a pretty lightweight operation. We may want to register it
# on the cloud at some point, if further tracking and logging is needed.
#
# We will also be revving this further when dealing with fine-grained accounts
# and roles.
#
import click
from simpleiot.common.utils import *
from simpleiot.common.config import *

from rich import print
from rich.console import Console
from rich.table import Table
import boto3
import questionary
from cryptography.fernet import Fernet
import base64
import time
from operator import itemgetter


console = Console()


@click.group()
def team():
    """Team management (CLI only)"""


#
# List displays all the teams in the current system. It has both short and full versions.
# It lists teams, based on what's available in the ~/.simpleiot directory. If a team name is specified
# we will display the full version.
#
@team.command()
@click.option("--team", "--name", help="Team name")
def list(team):
    """
    List known teams
    """
    try:
        if team:
            config = load_config(team)
            if not config:
                print(f"ERROR: team [ {team} ] not found.")
                exit(0)

            data = {
                "Team Name": config.get("team", "***"),
                "Organization": config.get("org_name", "***"),
                "AWS Account #": config.get("account", "***"),
                "Admin Email": config.get("admin_email", "***"),
                "Dashboard URL": _linkify(config.get("web_endpoint", "***")),
                "REST API URL": _linkify(config.get("apiEndpoint", "***")),
                "IOT Endpoint": _linkify(config.get("iot_endpoint", "***")),
                "Date Installed": config.get("install_utc_time", "***")
            }
            show_detail(console, "Team", data)
        else:
            team_data = list_all_teams()
            ordered_list = sorted(team_data.keys())
            table = Table(show_header=True, header_style="green")
            table.add_column("Team", style="dim", overflow="flow")
            table.add_column("Org")
            table.add_column("Date Installed", justify="right")
            for one_team in ordered_list:
                data = team_data[one_team]
                install_date = data.get("install_utc_time", None)
                if install_date:
                    install_date = format_date(install_date)
                else:
                    install_date = "***"
                table.add_row(one_team, data["org_name"], install_date)

            console.print(table)
    except Exception as e:
        print(f"ERROR: can not list teams. {str(e)}")
        exit(1)

def _linkify(link):
    result = None
    if link == "***":
        result = link
    else:
        result = f"[link=\"{link}\"]{link}[/link]"
    return result

#
# Invite is used by the admin to send invites to other team members.
# It collects the bare minimum information that is needed to get someone else to join.
# As a simple security measure (until we get proper roles), it checks to see if the
# bootstrap file is present and has information that indicates the user is the
# installer.
#
# NOTE: this user must be added to the Cognito pool (or if SSO, we assume it
# exists.
#
@team.command()
@common_cli_params
@click.option("--username", "-user", help="Username", required=True)
@click.option("--email", "-email", help="User email", required=True)
@click.option("--alias", "-as", help="Invite Team Alias")
@click.option("--path", "-p", help="Output Path for Invite file", default=".")
def invite(team, profile, username, email, alias, path):
    """
    Registers a new user and generates an invite file.
    """
    try:
        config = preload_config(team, profile)
        account = _is_admin(config)
        if not account:
            print("ERROR: invites can only be sent out by the administrator.")
            exit(1)

        if not os.path.isdir(path):
            print(f"ERROR: output path [{path}] does not exist")
            exit(1)

        temp_password = _add_user(config, username, email)
        if temp_password:
            team_as = config.team
            if alias:
                team_as = alias
            output_file_prefix = f"invite_{team_as}_{username}"
            invite_path = os.path.join(path, f"{output_file_prefix}.simpleiot")
            key = _create_invite(config, account, invite_path, username, email, temp_password)
            if key:
                print(f"DONE:\nInvite Key: {key}\nFile: {invite_path}")
                print("\nSend files and key to recipient and have them join via CLI\n\n% iot team join --invite {filename}.")
            else:
                print(f"ERROR saving invite files to path: '{path}'")

    except Exception as e:
        print(f"ERROR: can not send invite. {str(e)}")
        exit(1)

#
# Add a user to Cognito. If successful, we return the temporary password generated
# for the account. If not, we return None and let the caller deal with it.
# If username is a dupe or the password is invalid, we signal it here and return None.
#
def _add_user(config, username, email):
    temp_password = None
    try:
        local_config = load_config(config.team)
        use_sso = local_config.get("use_sso", False)
        if not use_sso:
            region = local_config.get("region", None)
            user_pool_id = local_config.get("cognitoUserPoolId", None)
            profile = local_config.get("aws_profile", "default")
            os.environ['AWS_PROFILE'] = profile
            temp_password = generate_temp_password()
            cognito = boto3.client('cognito-idp', region_name=region)
            response = cognito.admin_create_user(
                UserPoolId=user_pool_id,
                Username=username,
                TemporaryPassword=temp_password,
                UserAttributes=[
                    {"Name": "email_verified", "Value": "true"},
                    {"Name": "email", "Value": email}
                ],
                MessageAction='SUPPRESS'
            )
            result = True
    except ClientError as e:
        temp_password = None
        code = e.response['Error']['Code']
        if code == "UsernameExistsException":
            print(f"ERROR: user {username} already exists.")
        elif code == "InvalidPasswordException":
            print(f"ERROR: password invalid. Please delete user and try again.")
        else:
            print(f"ERROR: {str(e)}")

    return temp_password

#
# This uses an invitation file filled with encrypted data needed to join.
# It returns the key if the invitation was created. The path in which we want the
# invitation is written is passed down. In single-invite mode, the caller can pass the
# name of the invite. In bulk-mode, the routine calling us will have created a temporary
# file in the tmp directory.
#
def _create_invite(config, account, invite_path, username, email, temp_password):
    result = None
    try:
        team = config.team
        local_config = load_config(team)
        use_sso = local_config.get("use_sso", False)
        if not use_sso:
            region = local_config.get("region", None)
            org_name = local_config.get("org_name", None)
            api_endpoint = local_config.get("apiEndpoint", None)
            iot_endpoint = local_config.get("iot_endpoint", None)
            cognito_identity = local_config.get("cognitoIdentityPoolId", None)
            cognito_client_id = local_config.get("cognitoClientId", None)
            cognito_user_pool_id = local_config.get("cognitoUserPoolId", None)
            dashboard_url = local_config.get("dashboardDomainName", None)

            mqtt_port = local_config.get("mqtt_port", 8883)
            payload = {
                "team": team,
                "region": region,
                "account": account,
                "username": username,
                "temp_password": temp_password,
                "email": email,
                "org_name": org_name,
                "use_sso": use_sso,
                "apiEndpoint": api_endpoint,
                "iot_endpoint": iot_endpoint,
                "mqtt_port": mqtt_port,
                "cognitoIdentityPoolId": cognito_identity,
                "cognitoClientId": cognito_client_id,
                "cognitoUserPoolId": cognito_user_pool_id,
                "dashboard_url": f"https://{dashboard_url}"
            }

        if use_sso:
            bootstrap = load_bootstrap_config(team)
            payload["sso_url"] = bootstrap.get("sso_url")
            username = "sso"

        key, payload_str = _encrypt_invite(payload)

        # This could also generate a QRCode file with above data.
        # To be used when logging in from mobile or something with a camera.
        #
        # write_qr_file(path, output_file_prefix, "qrcode", payload_str)
        #
        with open(invite_path, 'w', encoding='utf-8') as outfile:
            outfile.write(payload_str)

        result = key
    except Exception as e:
        print(f"ERROR: can not generate invite file. {str(e)}")

    return result


@team.command()
@common_cli_params
@click.option("--input", "-i", help="Input CSV file", default="invite.csv")
@click.option("--alias", "-as", help="Invite Team Alias")
@click.option("--mailconf", help="Email config file", default="simpleiotmail.conf")
@click.option("--via", help="Mailing service, default is 'ses'", default="ses")
def bulkinvite(team, profile, input, alias, mailconf, via):
    """
    Bulk user addition with email invitation.
    """
    try:
        config = preload_config(team, profile)

        import email, smtplib, ssl
        from email import encoders
        from email.mime.base import MIMEBase
        from email.mime.multipart import MIMEMultipart
        from email.mime.text import MIMEText
        import tempfile
        from configparser import ConfigParser
        import csv
        import os
        import shutil

        smtp_server = None
        smtp_port = None
        smtp_username = None
        smtp_password = None
        from_email = None
        server = None

        EMAIL_CIPHERS = ('ECDH+AESGCM:DH+AESGCM:ECDH+AES256:DH+AES256:ECDH+AES128:DH+AES:ECDH+HIGH:'
            'DH+HIGH:ECDH+3DES:DH+3DES:RSA+AESGCM:RSA+AES:RSA+HIGH:RSA+3DES:!aNULL:!eNULL:!MD5')
        team = config.team

        # First we do our check to see if this is on the admin machine.
        # TODO: if we move to a container-based installation, this test will have to be reevaluated.
        #
        account = _is_admin(config)
        if not account:
            print("ERROR: bulk invites can only be sent out by the system installer.")
            exit(1)

        # Check the config file. It should have at least the port/server/username/password
        # values.
        #
        if not os.path.exists(mailconf):
            print(f"ERROR: Bulk email configuration file [{mailconf}] not found.")
            exit(1)

        try:
            config_p = ConfigParser()
            with open(mailconf, encoding='utf-8') as email_input:
                #
                # We look for a section header in the config file that matches the 'via' keyword.
                # This way, they can switch to different email providers.
                # If none is specified, we may want to attach a default section
                #
                # default_config = "[default]\n" + email_input.read()
                # config_p.read_string(default_config)

                config_p.read_string(email_input.read())

            section = config_p[via]
            smtp_port = section.get('smtp_port', '587')
            smtp_server = section['smtp_server']
            smtp_username = section['smtp_username']
            smtp_password = section['smtp_password']
            from_email = section['from_email']

            if not (smtp_port and smtp_server and smtp_username and smtp_password and from_email):
                print(f"ERROR: SMTP email configuration values missing in '{mailconf}'")
                exit(1)

            # First we try to connect to the SMTP server. If it fails, we don't create extra
            # users.
            #
            try:
                print(f"Trying to to connect to mail server: {smtp_server} on port {smtp_port}")

                server = smtplib.SMTP(smtp_server, port=smtp_port)
                if not server:
                    print(f"ERROR: could not create a connection to the SMTP server {smtp_server} on port {smtp_port}")
                    print(f"       Please remove users and re-run again.")
                    exit(1)

                server.ehlo()  # send the extended hello to our server

                # only TLSv1 or higher
                context = ssl.SSLContext(ssl.PROTOCOL_SSLv23)
                context.options |= ssl.OP_NO_SSLv2
                context.options |= ssl.OP_NO_SSLv3

                context.set_ciphers(EMAIL_CIPHERS)
                context.set_default_verify_paths()
                context.verify_mode = ssl.CERT_REQUIRED

                if server.starttls(context=context)[0] != 220:
                    print("ERROR: could not connect securely to email server")
                    exit(1)

                print(f"Logging in to mail server with username: {smtp_username}.")
                server.login(smtp_username, smtp_password)

            except Exception as e:
                print(f"ERROR: Error connecting to server: {str(e)}")
                exit(1)

        except Exception as e:
            print(f"ERROR: Error loading section '{via}' in bulk email configuration file '{mailconf}'")
            exit(1)

        if not os.path.exists(input):
            print(f"ERROR: input CSV file [{input}] does not exist.")
            exit(1)

        invite_list = []
        temp_dir = tempfile.mkdtemp() # where we stage the inputs

        with open(input, newline='', encoding='utf-8') as csvfile:
            recipient_reader = csv.reader(csvfile)
            for row in recipient_reader:
                invite_email = unquote(row[0])
                if invite_email == "email":
                    continue
                invite_username = unquote(row[1])

                account = _is_admin(config)
                if not account:
                    print("ERROR: invites can only be sent out by the administrator.")
                    exit(1)

                temp_password = _add_user(config, invite_username, invite_email)
                if not temp_password:
                    print(f"ERROR: could not add user named '{invite_username}' -- skipping")
                    continue

                team_as = team
                if alias:
                    team_as = alias

                output_file_prefix = f"invite_{team_as}_{invite_username}"
                invite_filename = f"{output_file_prefix}.simpleiot"
                invite_path = os.path.join(temp_dir, invite_filename)
                key = _create_invite(config, account, invite_path, invite_username, invite_email, temp_password)
                if key:
                    invite = {
                        "username": invite_username,
                        "email": invite_email,
                        "invite_file": invite_filename,
                        "invite_path": invite_path,
                    }
                    invite_list.append(invite)

                    invite_body = f"""
                    You have been invited to join the '{team}' SimpleIOT Team!
                    
                    To join, you will need to:
                    
                    1) Install the SimpleIOT command-line interface. Instructions are at: http://simpleiot.net.
                    2) Save the attached invitation file to your local file system.
                    3) Copy/paste the temporary key below. You will be asked for it in the next step.
                    
                       {key}
                    
                    4) Invoke the command:
                    
                       iot join --invite={{path-to-invite-file}} --key='{key}'
                       
                    5) You will be asked to enter a password for your account. It should contain upper and lower case, digits, and a special character.
                    6) If successful, you will be shown your team name and the link to your team's web dashboard.
                    7) You can login to the dashboard with your username '{invite_username}' and your chosen password.
                    8) For command-line access, please login with:
                    
                       iot login --team={team_as} --username={invite_username}
                       
                    You will be prompted for your chosen password.
                    
                    Congratulations! Now you are ready to use the SimpleIOT system.
                    
                    Please proceed with the workshop or tutorial to see how easy it is to create a cloud-connected devices.
                    """
                    invite["body"] = str(invite_body)
                else:
                    print(f"ERROR saving invite files to path: '{invite_path}' -- skipping")

                if server:
                    for one in invite_list:
                        recipient_email = one["email"]
                        print(f"Sending email invite to: '{recipient_email}'")
                        message = MIMEMultipart()
                        message["From"] = from_email
                        message["To"] = recipient_email
                        message["Subject"] = "SimpleIOT Invite"
                        message.attach(MIMEText(one["body"], "plain"))

                        # Open PDF file in binary mode
                        invite_path = one["invite_path"]
                        with open(invite_path, "rb") as attachment:
                            part = MIMEBase("application", "octet-stream")
                            part.set_payload(attachment.read())

                        # Encode file in ASCII characters to send by email
                        encoders.encode_base64(part)

                        # Add header as key/value pair to attachment part
                        filename = one["invite_file"]
                        part.add_header(
                            "Content-Disposition",
                            f"attachment; filename= {filename}"
                        )
                        message.attach(part)
                        text = str(message.as_string())

                        rec_user = one["username"]
                        rec_email = one["email"]
                        server.sendmail(from_email, rec_email, text)
                        print(f"Email sent to user {rec_user} at {rec_email}")
                        time.sleep(1)

        shutil.rmtree(temp_dir)

    except ClientError as e:
        code = e.response['Error']['Code']
        if  code == "UsernameExistsException":
            print(f"ERROR: user {username} already exists.")
        elif code == "InvalidPasswordException":
            print(f"ERROR: password invalid. Please delete user and try again.")
        else:
            print(f"ERROR: {str(e)}")
    except Exception as e:
        print(f"ERROR: can not send invite. {str(e)}")
        exit(1)
    finally:
        if server:
            server.quit()

#
# NOTE: for joining, we don't need the --team parameter specified.
#

@team.command()
@click.option("--invite", help="JSON invitation file", type=click.Path(exists=True), required=True)
@click.option("--key", "-key", help="Invite Decode key")
@click.option("--password", "-pass", help="New account password")
@click.option("--alias", "-as", help="Invite Team name override")
def join(invite, key, password, alias):
    """
    Join an existing team, given invite credentials.
    \f
    This command lets the user join a specific team by downloading the
    credentials necessary to remotely interact with projects, models, etc.

    Byt the time this invite has been sent, the user will have been created
    (if in Cognito) and/or added to the Group in SSO.

    Joining will create a local config file with enough info to let the user
    run CLI operations. To be able to do GUI stuff, they need to login with
    the credentials inside the file (username/temp password).

    At this point we're not doing fine-grain access control, but leaving room to
    enable access only to specific projects.

    :param config:
    :param invite: JSON invitation file sent by the administrator.
    :param alias: instead of the team in the invite, let's join under a different team name.
    :return:

    Examples:
    \b
    $ iot team join --invite myinvite.json

    """
    try:
        invite_text = None
        username = None
        temp_password = None

        with open(invite, "r", encoding='utf-8') as infile:
            invite_text = infile.read()

        if not invite_text:
            print(f"ERROR reading invite file: {invite}")
            exit(1)

        while not key:
            key = questionary.password(f"Enter invite key:").ask()
            if not key:
                print("Please enter the invite key from your invitation message")

        # Make sure we strip out any extra quotes from the ends
        key = unquote(key)

        invite_data_json = _decrypt_invite(key, invite_text)
        invite_data = json.loads(invite_data_json)

        # Invites only work for non-SSO teams. For SSO we assume they have their own
        # invitation mechanism and workflow.
        #
        use_sso = invite_data.get("use_sso", None)
        if not use_sso:
            #
            # We load up the username to cache, but we don't want to save it to the
            # config file. However, we give the user a chance to log in with these credentials.
            # NOTE: we're only using admin login for now. This is in place when we switch over to
            # multi-user support.
            #
            username = invite_data.get("username", None)

        if alias:
            team = alias
        else:
            team = invite_data.get("team", None)

        if team:
            if does_team_exist(team):
                print(f"ERROR: team '{team} already exists on your system.\n       Try adding '--alias={{team-name}}' to choose a different name.")
                exit(1)

            proceed = questionary.confirm(f"Would you like to join the SimpleIOT team [{team}]? ").ask()
            if proceed:
                # Ask for password (using questionary) if not set
                temp_password = invite_data.get("temp_password", None)
                while not password:
                    password = questionary.password(f"New Password:").ask()
                    if not password:
                        print("Please enter password")

                # We remove the username and temporary password from the invite. No need to store it
                # permanently.
                #
                del invite_data['username']
                del invite_data['temp_password']

                config_path = path_for_config_file(team)
                config_data_str = json.dumps(invite_data, indent=4)
                with open(config_path, 'w', encoding='utf-8') as configfile:
                    configfile.write(config_data_str)

                #
                # Now we do the password reset flow using Cognito
                #
                region = invite_data.get("region", None)
                client_id = invite_data.get("cognitoClientId", None)
                if region and client_id:
                    cognito = boto3.client('cognito-idp', region_name=region)
                    response = cognito.initiate_auth(
                        ClientId=client_id,
                        AuthFlow='USER_PASSWORD_AUTH',
                        AuthParameters={
                            'USERNAME': username,
                            'PASSWORD': temp_password
                        }
                    )
                    user_pool_id = invite_data.get('cognitoUserPoolId', None)
                    challenge = response.get("ChallengeName", None)
                    if challenge == "NEW_PASSWORD_REQUIRED":
                        password = unquote(password)
                        challenge_response = cognito.respond_to_auth_challenge(
                            ClientId=client_id,
                            ChallengeName=response['ChallengeName'],
                            Session=response['Session'],
                            ChallengeResponses={
                                'USERNAME': username,
                                'NEW_PASSWORD': password
                            }
                        )
                    else:
                        print(f"Password for user {username} has already been changed.")
                        return

                print("SIMPLEIOT: Team Added.")
                print(" - Make sure you set an environment variable for IOT_TEAM")
                print("   Then visit dashboard in browser or 'iot auth login' in console to login")
                print(f"     with your username: {username} and chosen password.")
                dashboard = invite_data.get("dashboard_url", None)
                table = Table(show_header=True, header_style="green")
                table.add_column("Name", style="dim", overflow="flow")
                table.add_column("Value", style="dim", overflow="flow")
                table.add_row("Dashboard", dashboard)
                if not use_sso:
                    table.add_row("Username", username)

                console.print(table)
        else:
            print(f"ERROR: no team specified")
            exit(1)

    except Exception as e:
        print(f"ERROR: could not load invite data from[{invite}]: {str(e)}")
        exit(1)


@team.command()
@common_cli_params
@click.option("--username", "-user", help="Username", required=True)
def leave(team, profile, username):
    """
    Deletes a user's login credential.
    """
    try:
        config = preload_config(team, profile)
        cognito = None
        account = _is_admin(config)
        if not account:
            print("ERROR: users can only be managed by the administrator.")

        if not ask_to_confirm_yesno("Are you sure you want to delete this user"):
            return

        team = config.team
        if team:
            bootstrap = load_bootstrap_config(team)
            if bootstrap:
                account = bootstrap.get("account", None)
                if not account:
                    print("ERROR: invites can only be sent out by the administrator.")
                    exit(1)
                else:
                    local_config = load_config(team)
                    _remove_user(config, local_config, team, username)
                    # If it fails, it will throw an exception that will be caught later
                print(f"Done: user {username} deleted.")
            else:
                print("ERROR: users can only be delete by the administrator.")
                exit(1)
        else:
            print("ERROR: no team specified")
            exit(1)

    except ClientError as e:
        code = e.response['Error']['Code']
        if code == "UserNotFoundException":
            print(f"ERROR: user {username} not found.")
        else:
            print(f"ERROR: {str(e)}")
    except Exception as e:
        print(f"ERROR: can not delete user. {str(e)}")
        exit(1)


# If it fails, it will throw an exception that will be caught later
#
def _remove_user(config, local_config, team, username):
    use_sso = local_config.get("use_sso", False)
    if not use_sso:
        region = local_config.get("region", None)
        user_pool_id = local_config.get("cognitoUserPoolId", None)
        profile = local_config.get("aws_profile", "default")
        os.environ['AWS_PROFILE'] = profile
        cognito = boto3.client('cognito-idp', region_name=region)
        response = cognito.admin_delete_user(
            UserPoolId=user_pool_id,
            Username=username)


#
# We use this to encrypt the invite payload and turn it into BASE64 text. The invite
# text and the resulting 'key' are then returned. The user should be sent both the invite file
# as well as the key. When they go to join, they will be entering the path to the invite as well
# as the 'key' text. This decrypts the payload, loads what's needed, and prompts the user to enter
# their own unique password for their account.
#
def _encrypt_invite(payload_json):
    try:
        key = Fernet.generate_key()
        f = Fernet(key)
        json_str = json.dumps(payload_json)
        encrypted_payload = f.encrypt(json_str.encode('utf-8'))
        b64_encrypted_payload = base64.standard_b64encode(encrypted_payload).decode('utf-8')
        b64_key = base64.standard_b64encode(key).decode('utf-8')
        return b64_key, b64_encrypted_payload
    except Exception as e:
        print(f"Error encrypting invite: {str(e)}")
        exit(1)


def _decrypt_invite(b64_key, b64_encrypted_payload_str):
    try:
        keyb = b64_key.encode('utf-8')
        key = base64.standard_b64decode(keyb)
        f = Fernet(key)
        b64_encrypted_payload = base64.standard_b64decode(b64_encrypted_payload_str)
        payload = f.decrypt(b64_encrypted_payload)
        return payload
    except Exception as e:
        print(f"Error decrypting invite: {str(e)}")
        exit(1)


def _is_admin(config):
    response = None
    try:
        team = config.team
        if team:
            bootstrap = load_bootstrap_config(team)
            if bootstrap:
                account = bootstrap.get("account", None)
                if account:
                    response = account
    except Exception as e:
        pass

    return response



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.common.utils import make_api_request, show_detail
from simpleiot.common.config import *

from rich import print
from rich.console import Console
from rich.table import Table
import requests

console = Console()

@click.group()
def data():
    """Data set and retrieve"""

@data.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Device Serial number", required=True)
@click.option("--name", "--type", help="Data name", default='')
@click.option("--desc", help="Data Description", default='')
@click.option("--value", help="Data value", default='')
@click.option("--data", help="Data series (name=value,name=value,...)", default='')
@click.option("--position", help="Data position (lat, long, alt)", default='')
@click.option("--dimension", help="Data dimension", default='')
def set(team, profile, project, serial, name, desc, value, data, position, dimension):
    """Set value for a given data element
    \f
    Examples:

    $ iot data set --project "..." --serial "..." --name "..." --value "..."
    $ iot data set --project "..." --serial "..." --data="name=value,name-value,..." ...

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    Doing so lets you skip the --project flag.

    To specify a data series (i.e. multiple settings at the same time, use the "--data"
    parameter. Values provided with the --name/--value parameter are then ignored.
    """
    try:
        config = preload_config(team, profile)

        if data:
            dataparams = {}
            datalist = data.split(",")
            for one in datalist:
                kvlist = one.split("=")
                try:
                    if len(kvlist) == 2:
                        n = kvlist[0].strip()
                        v = kvlist[1].strip()
                        dataparams[n] = v
                except Exception as e:
                    print(f"Invalid data parameter in [ {kvlist} ]. Skipping.")
                    pass
                data_str = ','.join('{}={}'.format(k,v) for k,v in dataparams.items())
            payload = {
                "project": project,
                "serial": serial,
                "data": data_str
            }
        else:
            payload = {
                "project": project,
                "serial": serial,
                "name": name,
                "value": value
            }

        if desc:
            payload["desc"] = desc
        if position:
            payload["position"] = position
        if dimension:
            payload["dimension"] = dimension

        response = make_api_request("POST", config, "data", json=payload)

        data = response.json()
        if response.status_code == requests.codes.ok:
            table = Table(show_header=True, header_style="green")
            table.add_column("Data ID", style="dim", overflow="flow")
            table.add_column("Project")
            table.add_column("Serial")
            table.add_column("Name")
            table.add_column("Value")
            ret_data = data.get("data", None)
            for item in ret_data:
                project_id = item.get("id", "***")
                serial = item.get("serial", "***")
                name = item.get("name", "***")
                value = item.get("value", "***")
                project_name = item.get("project", "***")
                table.add_row(project_id, project_name, serial, name, value)

        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Data Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@data.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Serial number", required=True)
@click.option("--name", "--type", help="Data name", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def get(team, profile, project, serial, name, full):
    """Return last known value for a given data element
    \f
    Examples:

    $ iot data get --project "..." --serial "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        multi = False

        if not name:
            print("Missing data name")
            return

        response = make_api_request("GET", config, f"data?project={project}&serial={serial}&name={name}")

        data = response.json()
        if response.status_code == requests.codes.ok:
            if full and not multi:
                show_detail(console, "Data", data)
                return

            table = Table(show_header=True, header_style="green")
            table.add_column("Data ID", style="dim", overflow="flow")
            table.add_column("Project")
            table.add_column("Serial")
            table.add_column("Name")
            table.add_column("Value")
            table.add_column("Timestamp", justify="right")
            project_id = data.get("id", "***")
            serial = data.get("serial", "***")
            name = data.get("name", "***")
            value = data.get("value", "***")
            timestamp = data.get("timestamp", "***")
            table.add_row(project_id, project, serial, name, value, timestamp)
        else:
            status = data.get("status", "")
            message = data.get("message", "")
            table = Table(show_header=True, header_style="red")
            table.add_column("Data List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@data.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Serial number", required=True)
@click.option("--name", help="Data name", default=None)
def delete(team, profile, project, serial, name):
    """Delete a data item already in the system
    \f
    Examples:

    $ iot data delete --project "..." --serial "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        response = None

        if name:
            response = make_api_request("DELETE", config, f"data?project_name={project}&serial={serial}&name={name}")

        data = response.json()

        if response.status_code == requests.codes.ok:
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
from simpleiot.common.utils import make_api_request, show_detail, format_date
from simpleiot.common.config import *

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table
import requests

console = Console()


@click.group()
def datatype():
    """Model DataType management"""


@datatype.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--name", help="DataType name", required=True)
@click.option("--desc", help="DataType Description", default="")
@click.option("--type", "--data_type", help="DataType type (i.e. integer, string, float)", default="integer")
@click.option("--units", help="DataType type units", default="")
@click.option("--show_on_twin", help="DataType shows on twin", type=bool, default=True)
@click.option("--data_position", help="DataType Twin position", default="")
@click.option("--data_normal", help="DataType Twin normal", default="")
@click.option("--label_template", help="DataType Twin label template", default="")
@click.option("--ranges", help="DataType value ranges JSON", default="")
@click.option("--rangefile", help="DataType value ranges JSON file", default="")
def add(team, profile, project, model, name, desc, type, units, show_on_twin,
        data_position, data_normal, label_template, ranges, rangefile):
    """Define a new Model DataType
    \f
    Examples:

    $ iot datatype add --name "..." --project "..." --model "..."

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    To specify a default model, set the environment variable IOT_MODEL to name of model.
    Doing so lets you skip the --project and --model flag.
    """
    try:
        config = preload_config(team, profile)

        payload = {
            "project_name": project,
            "model": model,
            "name": name
        }
        if desc:
            payload["desc"] = desc
        if type:
            payload["data_type"] = type
        if units:
            payload["units"] = units
        if show_on_twin:
            payload["show_on_twin"] = show_on_twin
        if data_position:
            payload["data_position"] = data_position
        if data_normal:
            payload["data_normal"] = data_normal
        if label_template:
            payload["label_template"] = label_template
        if rangefile:
            try:
                with open(rangefile, "r") as rf:
                    rangedata = rf.read()
                if rangedata:
                    payload["ranges"] = rangedata
            except:
                pass
        if ranges:
            payload["ranges"] = ranges

        response = make_api_request("POST", config, "datatype", json=payload)
        data = response.json()

        if response.status_code == requests.codes.ok:
            table = Table(show_header=True, header_style="green")
            table.add_column("DataType ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            project_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table.add_row(project_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@datatype.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--name", help="DataType name", default=None)
@click.option("--id", help="Model ID", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def list(team, profile, project, model, name, id, full):
    """Return information on defined Datatypes
    \f
    Examples:
    \b
    $ iot datatype list --project "..." --model "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        multi = False

        if name:
            response = make_api_request("GET", config, f"datatype?project={project}&model={model}&name={name}")
        elif id:
            response = make_api_request("GET", config, f"datatype?project_name={project}&model={model}&id={id}")
        else:
            multi = True
            response = make_api_request("GET", config, f"datatype?project_name={project}&model={model}&all=true")

        data = response.json()
        if response.status_code == requests.codes.ok:
            if full and not multi:
                show_detail(console, "Datatype", data)
                return

            table = Table(show_header=True, header_style="green")
            table.add_column("DataType ID", style="dim", overflow="flow")
            table.add_column("Project")
            table.add_column("Model")
            table.add_column("Name")
            table.add_column("Type")
            table.add_column("Units")
            table.add_column("Date Created", justify="right")
            if multi:
                for d in data:
                    id = d.get("id", "***")
                    project = d.get("project", "***")
                    model = d.get("model", "***")
                    name = d.get("name", "***")
                    data_type = d.get("data_type", "***")
                    units = d.get("units", "***")
                    created = d.get("date_created", "***")
                    table.add_row(id, project, model, name, data_type, units, format_date(created))
            else:
                id = data.get("id", "***")
                project = data.get("project", "***")
                model = data.get("model", "***")
                name = data.get("name", "***")
                data_type = data.get("data_type", "***")
                units = data.get("units", "***")
                created = data.get("date_created", "***")
                table.add_row(id, project, model, name, data_type, units, format_date(created))
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Model List Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@datatype.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--id", help="Model ID", default=None)
@click.option("--name", help="Datatype name", default=None)
@click.option("--desc", help="DataType description", default="")
@click.option("--type", "--data_type", help="DataType type (i.e. integer, string, float)", default="integer")
@click.option("--units", help="DataType type units", default="")
@click.option("--show_on_twin", help="DataType shows on twin", type=bool, default=False)
@click.option("--data_position", help="DataType Twin position", default="")
@click.option("--data_normal", help="DataType Twin normal", default="")
@click.option("--label_template", help="DataType Twin label template", default="")
@click.option("--ranges", help="DataType value ranges JSON", default="")
@click.option("--rangefile", help="DataType value ranges JSON file", default="")
def update(team, profile, project, model, id, name, desc, type, units,
           show_on_twin, data_position, data_normal, label_template,
           ranges, rangefile):
    """
    Update settings for a given Datatype
    \f
    Updates DeviceType attributes. NOTE: a DataType's project and model can not be modified.
    To change it, delete it then re-add with the new model.

    Examples:

    $ iot datatype update --name "..." --project "..."

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    To specify a default model, set the environment variable IOT_MODEL to name of model.
    Doing so lets you skip the --project and --project flags.
    """
    try:
        config = preload_config(team, profile)

        if not name and not id:
            print(f"Error: model 'name' or 'id' must be specified")
            exit(1)

        payload = {
            "project_name": project,
        }
        if model:
            payload["model"] = model
        if name:
            payload["name"] = name
        elif id:
            payload["id"] = id

        if desc:
            payload["desc"] = desc
        if type:
            payload["type"] = type
        if units:
            payload["units"] = units
        if show_on_twin:
            payload["show_on_twin"] = show_on_twin
        if data_position:
            payload["data_position"] = data_position
        if data_normal:
            payload["data_normal"] = data_normal
        if label_template:
            payload["label_template"] = label_template
        if rangefile:
            try:
                with open(rangefile, "r") as rf:
                    rangedata = rf.read()
                if rangedata:
                    payload["ranges"] = rangedata
            except:
                pass
        if ranges:
            payload["ranges"] = ranges

        url_params = urllib.parse.urlencode(payload)
        url = f"datatype?{url_params}"
        response = make_api_request("PUT", config, url)
        data = response.json()

        if response.status_code == requests.codes.ok:
            model_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("DataType Update ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(model_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@datatype.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--name", help="DataType name", default=None)
@click.option("--id", help="Project ID", default=None)
def delete(team, profile, project, model, name, id):
    """
    Delete an existing Datatype definition
    \f
    Examples:
    \b
    $ iot model delete --project "..." --model "..." --name "..."
    """
    try:
        config = preload_config(team, profile)

        response = None

        if name:
            response = make_api_request("DELETE", config, f"datatype?project_name={project}&model={model}&name={name}")
        elif id:
            response = make_api_request("DELETE", config, f"datatype?project_name={project}&model={model}&id={id}")

        data = response.json()

        if response.status_code == requests.codes.ok:
            id = data.get("id", "")
            status = data.get("status", "")
            message = data.get("message", "")
            table = Table(show_header=True, header_style="bold green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# Cloud installation:
#
# High-level:
#   install
#   update
#   uninstall
#   version
#   info
#
# Low-level:
#   terminal - used to shell into the container -- for debugging only.
#
import platform

import click
import os
import json
import time
from simpleiot.common.utils import *
from simpleiot.common.config import *
import signal
import sys
import subprocess
import docker

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table
import requests
from pathlib import Path


DOCKER_IMAGE = "amazon/simpleiot-installer"
LOCAL_OUT_DIR = os.path.join(tempfile.gettempdir(), "simpleiot-layer")
SAVED_TEAM_NAME_FILE = "simpleiot_last_install_team.txt"

console = Console()

#
# This checks to see if the ~/.simpleiot directory exists or not. If not, it creates it so the
# docker mapping to the directory works.
#
def create_settings_if_not_exist():
    abs_path = os.path.expanduser("~/.simpleiot")
    if not os.path.exists(abs_path):
        os.mkdir(abs_path)

def clean_windows_path(src):
    split_path = os.path.splitdrive(src)
    prefix = (split_path[0][:1])
    suffix = Path(split_path[1]).as_posix()
    full_path = f"/{prefix}{suffix}"
    return full_path


def _invoke_unbuffered(command):
    try:
        if sys.platform == "win32":
            # For windows we use wexpect
            import msvcrt

            msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
            os.system(command)

        else:
            result = os.system(command)
            print(result)
            #
            # # For mac or Linux we use pexpect
            # import subprocess
            # result = subprocess.check_output(command, shell=True, stderr=subprocess.STDOUT)
            # print(result)

            # import pexpect
            #
            # c = pexpect.spawnu(command)
            # c.interact()
            # c.kill(1)

    except Exception as e:
        pass
    

def is_docker_on():
    result = False
    try:
        client = docker.from_env();
        client.ping()
        result = True
    except Exception as e:
        pass
    return result
#
# This command is a template for running the actual command inside the docker container
# that we have already installed. If not installed,
#
def run_in_docker(cmd, param=None, team=None):
    #
    # First, let's check and see if docker daemon is running
    #
    if is_docker_on():
        create_settings_if_not_exist()
        abs_aws_path = os.path.expanduser(os.path.join("~", ".aws"))
        abs_simpleiot_path = os.path.expanduser(os.path.join("~", ".simpleiot"))
        if not os.path.exists(LOCAL_OUT_DIR):
            os.mkdir(LOCAL_OUT_DIR)

        out_dir = Path(LOCAL_OUT_DIR).as_posix()

        if sys.platform == 'win32':
            abs_aws_path = clean_windows_path(abs_aws_path)
            abs_simpleiot_path = clean_windows_path(abs_simpleiot_path)
            out_dir = clean_windows_path(LOCAL_OUT_DIR)

        command = f"{cmd}"
        if param:
            command = f"{command} {param}"
        if team:
            command = f"{command} {team}"
        _invoke_unbuffered(f"docker pull {DOCKER_IMAGE}:latest")

        # os.system(f"docker run -i \
        _invoke_unbuffered(f"docker run -i \
                --network host \
                -v /var/run/docker.sock:/var/run/docker.sock \
                --mount type=bind,source={abs_aws_path},target=/root/.aws \
                --mount type=bind,source={abs_simpleiot_path},target=/root/.simpleiot \
                --mount type=bind,source={out_dir},target=/opt/iotapi/iotcdk/lib/lambda_src/layers/iot_import_layer/out \
                -t {DOCKER_IMAGE}:latest {command}")
    else:
        print(f"ERROR: Docker daemon is not running. Please start Docker desktop, then try again.")
        exit(1)

# After the bootstrap stage, the name of the team is saved in a temp file for use in subsequent
# stages.
#
# Ordinarily, this would need to be deleted, but we save it during bootstrap so we can look up
# the user-designated team name and pass it on to the next stage.
#
def load_team_from_tempfile(filename=SAVED_TEAM_NAME_FILE):
    try:
        tempdir = tempfile.gettempdir()
        infile = os.path.join(tempdir, filename)
        if os.path.exists(infile):
            with open(infile, 'r') as out:
                result = out.read().replace('\n','')
                return result
        else:
            return None
    except Exception as e:
        print(f"ERROR: could not read data from tempfile {filename}")
        raise e


@click.group()
def cloud():
    """Cloud back-end provisioning"""


@cloud.command()
def install():
    """
    Install cloud back-end for a single Team

    This command invokes docker to load and run the three stages of the install
    process. It first runs bootstrap, which asks for the name of the team. That is saved
    in a fixed file in the shared settings volume. Then it runs deploy with that team
    name. Once that is done, it runs the dbsetup phase.

    If you want to run each phase separately, best to do these not from inside the
    CLI but download and run the installer from source.

    NOTE: we will want to be careful running this on top of an existing installation.
    If it already exists, bad things may happen.

    """
    try:
        status = "OK"
        message = ""
        print("Loading installer image...")
        run_in_docker("invoke", "install")

        table = Table(show_header=True, header_style="green")
        table.add_column("Install Status")
        table.add_column("Message")
        table.add_row(status, message)
        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@cloud.command()
@common_cli_params
def uninstall(team, profile):
    """Uninstall cloud back-end for a Team"""
    try:
        status = "OK"
        message = ""
        print("Loading uninstaller image...")
        run_in_docker("invoke", "clean", team)

        table = Table(show_header=True, header_style="green")
        table.add_column("Uninstall Status")
        table.add_column("Message")
        table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@cloud.command()
def terminal():
    """For debugging: connect to installer container terminal"""
    create_settings_if_not_exist()
    abs_aws_path = os.path.expanduser(os.path.join("~", ".aws"))
    abs_simpleiot_path = os.path.expanduser(os.path.join("~", ".simpleiot"))
    if not os.path.exists(LOCAL_OUT_DIR):
        os.mkdir(LOCAL_OUT_DIR)

    out_dir = Path(LOCAL_OUT_DIR).as_posix()

    if sys.platform == 'win32':
        abs_aws_path = clean_windows_path(abs_aws_path)
        abs_simpleiot_path = clean_windows_path(abs_simpleiot_path)
        out_dir = clean_windows_path(LOCAL_OUT_DIR)

    # TODO: If the --user flag is needed on Windows, we need to extract the uid:gid using Windows APIs. getpwuid is a POSIX function.
    #
    if sys.platform == 'win32':
        user_flag = ""
    else:
        import pwd

        suid = pwd.getpwuid(os.getuid())
        uid = suid.pw_uid
        gid = suid.pw_gid
        user_flag = f"--user {uid}:{gid}"

    os.system(f"docker run -i \
            --network host \
            {user_flag} \
            -v /var/run/docker.sock:/var/run/docker.sock \
            --mount type=bind,source={abs_aws_path},target=/root/.aws \
            --mount type=bind,source={abs_simpleiot_path},target=/root/.simpleiot \
            --mount type=bind,source={out_dir},target=/opt/iotapi/iotcdk/lib/lambda_src/layers/iot_import_layer/out \
            -t {DOCKER_IMAGE}:latest /bin/bash")





#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
import click
import json
import time
from simpleiot.common.utils import *
from simpleiot.common.config import *
import signal
import sys

import urllib, urllib.parse
from rich import print
from rich.console import Console
from rich.table import Table
import requests

console = Console()


@click.group()
def device():
    """Device provisioning"""


@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Device model", envvar="IOT_MODEL", required=True)
@click.option("--serial", help="Device Serial", required=True)
@click.option("--name", help="Device name", default="")
@click.option("--desc", help="Device Description", default="")
@click.option("--position", help="Device Position", default="")
@click.option("--latitude", help="Device Latitude", type=float, default=None)
@click.option("--longitude", help="Device Longitude", type=float, default=None)
@click.option("--altitude", help="Device Altitude", type=float, default=None)
@click.option("--status", help="Device Status", default=None)
@click.option("--error", help="Device Error Message", default=None)
@click.option("--arduino", help="Name of arduino project to generate", default=None)
@click.option("--wifi_ssid", "--ssid", help="Wifi SSID name", envvar="IOT_WIFI_SSID")
@click.option("--wifi_password", "--password", help="Wifi Password", envvar="IOT_WIFI_PASSWORD")

def add(team, profile, project, model, serial, name, desc,
        position, latitude, longitude, altitude,
        status, error, arduino, wifi_ssid, wifi_password):
    """
    Provision a single device
    \f
    Adds a new Device to the system. Project, Model, and Serial Number are required.
    Examples:
    \b
    $ iot device add --project "..." --model "..." --serial "..." ...

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    To specify a default model, set the environment variable IOT_MODEL to name of model.
    Doing so lets you skip the --project and --model flags.
    """
    try:
        config = preload_config(team, profile)

        payload = {
            "project_name": project,
            "model": model,
            "serial": serial
        }
        qr_data = json.dumps(payload)

        if name:
            payload["name"] = name
        if desc:
            payload["desc"] = desc
        if position:
            payload["position"] = position
        if latitude:
            payload["latitude"] = latitude
        if longitude:
            payload["longitude"] = longitude
        if altitude:
            payload["altitude"] = altitude
        if status:
            payload["status"] = status
        if error:
            payload["error"] = error

        response = make_api_request("POST", config, "device", json=payload)

        data = response.json()
        if response.status_code == requests.codes.ok:
            #
            # This creates the right local device cache directory, if needed
            #
            device_dir = get_iot_device_dir(config.team, project, model, serial, create=True)

            ca_data = data.get("ca_pem", None)
            if ca_data:
                write_device_cert_file(device_dir, serial, "rootca", ca_data)
            cert_data = data.get("cert_pem", None)
            write_device_cert_file(device_dir, serial, "cert", cert_data)
            public_key = data.get("public_key", None)
            write_device_cert_file(device_dir, serial, "public", public_key)
            private_key = data.get("private_key", None)
            write_device_cert_file(device_dir, serial, "private", private_key)

            table = Table(show_header=True, header_style="green")
            table.add_column("Device ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            device_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table.add_row(device_id, status, message)
        else:
            data = response.json()
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("List Status")
            table.add_column("Message")
            table.add_row(status, message)
        console.print(table)

        iot_endpoint = config.get("iot_endpoint", "***")
        print(f"Certificate directory: {device_dir}")
        print(f"IOT Endpoint: {iot_endpoint}")

        ## NOTE: this could be a generic output with device settings, so it invokes the toolchainbase class type
        # and lets *THAT* do the code generation. This way, each toolchain can auto generate its own skeleton
        # code and spit it out the way it's meant to.
        # NOTE 2: The firmware build/flash command should accept a folder instead of a zip file, so it can
        # compile and flash from an existing directory.
        #

        # If the arduino value is set, we will generate a full arduino project with a directory defined as the arduino
        # name, an .ino file that includes the proper files and sets the settings, the certs and wifi files
        # and a skeleton project that includes code to set the Datatypes defined for the project. All you have to do
        # is go wire up the sensors to the right variables and it's good to go.
        #
        if arduino:
            pass

        #print("NOTE: If this is a gateway, be sure to finish GG set-up from the device itself.")
    except Exception as e:
        print(f"ERROR: {str(e)}")

def write_device_cert_file(base, serial, suffix, data):
    try:
        if data:
            filename = f"{serial}_{suffix}.pem"
            file = os.path.join(base, filename)
            with open(file, "w") as out:
                out.write(data + '\n')
    except Exception as e:
        print(
            f"ERROR: unable to save device data for {serial} to {base} directory. Please delete device, fix problem, and try again")

@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", default=None)
@click.option("--model", help="Device model", envvar="IOT_MODEL", default=None)
@click.option("--serial", help="Device Serial", envvar="IOT_DEVICE_SERIAL", default=None)
@click.option("--id", help="Device ID", default=None)
@click.option("--location", help="Location Name", default=None)
@click.option("--location_id", help="Location ID", default=None)
@click.option("--full/--no-full", help="List Full Data", default=False)
def list(team, profile, project, model, serial, id, location, location_id, full):
    """
    Show information on one or more devices
    \f
    Lists information on device instances already in system.
    Examples:
    \b
    $ iot device list --project "..." ...
    """
    try:
        config = preload_config(team, profile)

        multi = False

        # If there's a serial, we explicitly show the one device
        if location:
            multi = True
            response = make_api_request("GET", config, f"device?project_name={project}&location={location}")
        elif location_id:
            response = make_api_request("GET", config, f"device?project_name={project}&location_id={location_id}")
        elif project and serial:
            response = make_api_request("GET", config, f"device?project_name={project}&serial={serial}")
        # Or a GUID
        elif project and id:
            response = make_api_request("GET", config, f"device?project_name={project}&id={id}")
        # If just a model, we show all devices of that model
        elif project and model:
            multi = True
            response = make_api_request("GET", config, f"device?project_name={project}&model={model}")
        # Otherwise, all devices for this project
        elif project:
            multi = True
            response = make_api_request("GET", config, f"device?project_name={project}&all=true")
        else:
            print("ERROR: insufficient paramters. Need at least project or location")
            exit(1)

        data = response.json()
        if response.status_code == requests.codes.ok:
            if full and not multi:
                show_detail(console, "Device", data)
                return

            table = Table(show_header=True, header_style="bold green")
            table.add_column("Device ID", style="dim", overflow="flow")
            if location or location_id:
                table.add_column("Location")
            else:
                table.add_column("Model")

            table.add_column("Serial")
            table.add_column("Name")
            table.add_column("Date Created", justify="right")
            if multi:
                for d in data:
                    id = d.get("id", "***")
                    model = d.get("model", "***")
                    serial = d.get("serial", "***")
                    name = d.get("name", "***")
                    created = d.get("date_created", "***")

                    if location or location_id:
                        table.add_row(id, location, serial, name, format_date(created))
                    else:
                        table.add_row(id, model, serial, name, format_date(created))
            else:
                id = data.get("id", "***")
                model = data.get("model", "***")
                serial = data.get("serial", "***")
                name = data.get("name", "***")
                created = data.get("date_created", "***")
                table.add_row(id, model, serial, name, format_date(created))
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Device Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")

@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Device model", envvar="IOT_MODEL", default=None)
@click.option("--serial", help="Device Serial", envvar="IOT_DEVICE_SERIAL", default=None)
@click.option("--id", help="Device ID", default=None)
@click.option("--output", help="Write output config file to this file", default='iotconfig.json')
def getconfig(team, profile, project, model, serial, id, output):
    """
    Generate iotconfig.json file for use in SimpleIOT clients
    \f
    Examples:
    \b
    $ iot device getconfig --project "..." ...
    """
    config = preload_config(team, profile)

    # # If there's a serial, we explicitly show the one device
    # if serial:
    #     response = make_api_request('GET', config, f"device?project_name={project}&serial={serial}")
    # # Or a GUID
    # elif id:
    #     response = make_api_request('GET', config, f"device?project_name={project}&id={id}")
    # # If just a model, we show all devices of that model
    # elif model:
    #     multi = True
    #     response = make_api_request('GET', config, f"device?project_name={project}&model={model}")
    # # Otherwise, all devices for this project
    # else:
    #     multi = True
    #     response = make_api_request('GET', config, f"device?project_name={project}&all=true")
    #
    # data = response.json()
    #
    # if response:
    #     if full and not multi:
    #         show_detail(console, "Device", data)
    #         return
    #
    #     table = Table(show_header=True, header_style="bold green")
    #     table.add_column("Device ID", style="dim", overflow="flow")
    #     table.add_column("Project")
    #     table.add_column("Model")
    #     table.add_column("Serial")
    #     table.add_column("Name")
    #     table.add_column("Date Created", justify="right")
    #     if multi:
    #         for d in data:
    #             id = d['id']
    #             project = d['project']
    #             model = d['model']
    #             serial = d['serial']
    #             name = d.get('name', "")
    #             created = d['date_created']
    #             table.add_row(id, project, model, serial, name, created)
    #     else:
    #         id = data['id']
    #         project = data['project']
    #         model = data['model']
    #         serial = data['serial']
    #         name = data.get('name', "")
    #         created = data['date_created']
    #         table.add_row(id, project, model, serial, name, created)
    # else:
    #     status = data.get('status', "")
    #     message = data.get('message', "")
    #     table = Table(show_header=True, header_style="red")
    #     table.add_column("Device Status")
    #     table.add_column("Message")
    #     table.add_row(status, message)
    #
    # console.print(table)


@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--id", help="Device ID", default=None)
@click.option("--serial", help="Device Serial", required=True)
@click.option("--name", help="Model name", required=True)
@click.option("--desc", help="Model Description", default="")
@click.option("--position", help="Model Position", default="")
@click.option("--latitude", help="Device Latitude", type=float, default=None)
@click.option("--longitude", help="Device Longitude", type=float, default=None)
@click.option("--altitude", help="Device Altitude", type=float, default=None)
@click.option("--status", help="Device Status", default=None)
@click.option("--error", help="Device Error Message", default=None)
def update(team, profile, project, id, serial, name, desc, position,
           latitude, longitude, altitude, status,
           error):
    """
    Update individual device meta-data settings
    \f
    Updates Device attributes. NOTE: a device's project or serial can not be modified.
    To change it, delete it then re-add.

    Examples:
    \b
    $ iot device update --name "..." --project "..." --image

    To specify a default project, set the environment variable IOT_PROJECT to name of project.
    Doing so lets you skip the --project flag.
    """
    try:
        config = preload_config(team, profile)

        if not serial and not id:
            print(f"Error: device 'serial' or 'id' must be specified")
            exit(1)

        payload = {
            "project_name": project,
        }
        if serial:
            payload["serial"] = serial
        elif id:
            payload["id"] = id

        if name:
            payload["name"] = name
        if desc:
            payload["desc"] = desc
        if position:
            payload["position"] = position
        if latitude:
            payload["latitude"] = latitude
        if longitude:
            payload["longitude"] = longitude
        if altitude:
            payload["altitude"] = altitude
        if status:
            payload["status"] = status
        if error:
            payload["error"] = error

        url_params = urllib.parse.urlencode(payload)
        url = f"devicel?{url_params}"
        response = make_api_request('PUT', config, url)
        data = response.json()
        if response.status_code == requests.codes.ok:
            model_id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Device Update ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(model_id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Update Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Device serial", default=None)
@click.option("--id", help="Device ID", default=None)
def delete(team, profile, project, serial, id):
    """
    Remove and de-provision an existing device instance
    \f
    Deletes a device already in the system.
    Examples:

    $ iot device delete --project "..." --serial "..."
    $ iot device delete --project "..." --id "..."
    """
    try:
        config = preload_config(team, profile)

        response = None

        if serial:
            response = make_api_request("DELETE", config, f"device?project_name={project}&serial={serial}")
        elif id:
            response = make_api_request("DELETE", config, f"device?project_name={project}&id={id}")

        data = response.json()
        if response.status_code == requests.codes.ok:
            # project = data.get("project", None)
            model = data.get("model", None)
            if model:
                delete_iot_device_dir(config.profile, project, model, serial)
            else:
                print("API Error: did not return device model. Could not delete local device directory")

            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Deleted ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Delete Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")

# Attach/detach associates a node device with a gateway. We verify that the target is
# a gateway, then create an association both in the database as well as
# in Greengrass. This allows each node device to directly connect to the gateway
# if used for device discovery.
#
# Detach goes the other way, removes the attachment and removes the device from the
# Greengrass group.
#
# NOTE that devices can only be attached/detached to devices in the same project.
#
@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Device serial", default=None)
@click.option("--to", "--gateway", help="Gateway serial", default=None)
@click.option("--from_id", help="End Device ID", default=None)
@click.option("--to_id", help="Gateway Device ID", default=None)
def attach(team, profile, project, serial, to, from_id, to_id):
    """
    Attach and end-node device to a gateway
    \f
    Examples:

    $ iot device attach --project "..." --serial "..." --to "..."
    $ iot device attach --project "..." --id "..." --to "..."
    """
    try:
        config = preload_config(team, profile)

        response = None
        query = None

        if serial and to:
            query = f"device?op=attach&project={project}&device={serial}&gateway={to}"
        elif serial and to_id:
            query = f"device?op=attach&project={project}&device_id={from_id}&gateway_id={to_id}"
        elif from_id and to:
            query = f"device?op=attach&project={project}&device_id={from_id}&gateway={to}"
        elif from_id and to_id:
            query = f"device?op=attach&project={project}&device_id={from_id}&gateway_id={to_id}"

        if not query:
            print("ERROR: device and gateway need to be specified")
            exit(1)

        response = make_api_request("PUT", config, query)
        data = response.json()
        if response.status_code == requests.codes.ok:
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Attached ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Attach Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")

@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", help="Device serial", default=None)
@click.option("--to", "--gateway", help="Gateway serial", default=None)
@click.option("--from_id", help="End Device ID", default=None)
@click.option("--to_id", help="Gateway Device ID", default=None)
def detach(team, profile, project, serial, to, from_id, to_id):
    """
    Detach an end-device from a gateway
    \f
    Examples:

    $ iot device detach --project "..." --serial "..." --to "..."
    $ iot device detach --project "..." --id "..." --to "..."
    """
    try:
        config = preload_config(team, profile)

        response = None
        query = None

        if serial and to:
            query = f"device?op=detach&project={project}&device={serial}&gateway={to}"
        elif serial and to_id:
            query = f"device?op=detach&project={project}&device_id={from_id}&gateway_id={to_id}"
        elif from_id and to:
            query = f"device?op=detach&project={project}&device_id={from_id}&gateway={to}"
        elif from_id and to_id:
            query = f"device?op=detach&project={project}&device_id={from_id}&gateway_id={to_id}"

        if not query:
            print("ERROR: device and gateway need to be specified")
            exit(1)

        response = make_api_request("PUT", config, query)
        data = response.json()

        if response.status_code == requests.codes.ok:
            id = data.get("id", "***")
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Detached Device ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_column("Message")
            table.add_row(id, status, message)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Detach Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


# Place/remove associates a device with a pre-defined location. Locations have to
# be created ahead of time using "iot location add" and given a unique name.
#
# Remove goes the other way, removes the location attachment.
#
# NOTE that devices can only be placed/unplaced in the same project.
#
@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--serial", "--device", help="Device serial", default=None)
@click.option("--at", "--location", help="Location name", default=None)
@click.option("--device_id", help="Device ID", default=None)
@click.option("--location_id", help="Location ID", default=None)
def place(team, profile, project, serial, at, location_id, device_id):
    """Place an end-device at a known location
    \f
    Examples:

    $ iot device place --project "..." --serial "..." --at "..."
    """
    try:
        config = preload_config(team, profile)

        response = None
        query = None

        if serial and at:
            query = f"device?op=place&project={project}&device={serial}&location={at}"
        elif serial and location_id:
            query = f"device?op=place&project={project}&device={device}&location_id={location_id}"
        elif device_id and at:
            query = f"device?op=place&project={project}&device_id={device_id}&gateway={at}"
        elif device_id and location_id:
            query = f"device?op=place&project={project}&device_id={device_id}&location_id={location_id}"

        if not query:
            print("ERROR: device and location need to be specified")
            exit(1)

        response = make_api_request("PUT", config, query)
        data = response.json()
        if response.status_code == requests.codes.ok:
            status = data.get("status", "***")
            device_id = data.get("device", "***")
            location_id = data.get("location", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Device ID", style="dim", overflow="flow")
            table.add_column("Location ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_row(device_id, location_id, status)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Place Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")

@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", default=None)
@click.option("--serial", "--device", help="Device serial", default=None)
@click.option("--id", help="Device ID", default=None)
def remove(team, profile, project, serial, id):
    """
    Remove an end-device from a location
    \f
    Examples:

    $ iot device remove --project "..." --serial "..."
    """
    try:
        config = preload_config(team, profile)

        response = None
        query = None

        if project and serial:
            response = make_api_request("PUT", config, f"device?op=remove&project={project}&device={serial}")
        elif id:
            response = make_api_request("PUT", config, f"device?op=remove&device_id={id}")
        else:
            print("ERROR: Device --project and --serial OR --id has to be specified")
            exit(1)

        data = response.json()

        if response.status_code == requests.codes.ok:
            device = data.get("device", "***")
            location = data.get("location", "***")
            status = data.get("status", "***")
            table = Table(show_header=True, header_style="green")
            table.add_column("Device ID", style="dim", overflow="flow")
            table.add_column("Location ID", style="dim", overflow="flow")
            table.add_column("Status")
            table.add_row(device, location, status)
        else:
            status = data.get("status", "***")
            message = data.get("message", "***")
            table = Table(show_header=True, header_style="red")
            table.add_column("Place Status")
            table.add_column("Message")
            table.add_row(status, message)

        console.print(table)
    except Exception as e:
        print(f"ERROR: {str(e)}")


def on_error_callback(error):
    table = Table(show_header=True, header_style="red")
    table.add_column("Error Message", style="bold", overflow="flow")
    table.add_row(f"[red]{error}[/red]")
    console.print(table)


def on_data_callback(topic, payload, show_raw, stop_after_one):
    name = payload.get("name", "**no-name**")
    value = payload.get("value", "**no-name**")
    #print(f"{topic}: {name} = {value}")

    if show_raw:
        payload_str = json.dumps(payload, indent=2)
        print(payload_str)
    else:
        table = Table(show_header=True, header_style="green")
        table.add_column("Data", style="bold", overflow="flow")
        table.add_column("Value", style="bold", overflow="flow")
        print_args = [f"[magenta]{name}[/magenta]", f"[yellow]{value}[/yellow]"]
        geo_lat = payload.get("geo_lat", None)
        geo_lng = payload.get("geo_lng", None)
        geo_alt = payload.get("geo_alt", None)
        if geo_lat:
            table.add_column("GPS Lat", style="bold", overflow="flow")
            print_args.append(f"[cyan]{geo_lat}[/cyan]")
        if geo_lng:
            table.add_column("GPS Lng", style="bold", overflow="flow")
            print_args.append(f"[cyan]{geo_lng}[/cyan]")
        if geo_alt:
            table.add_column("GPS Alt", style="bold", overflow="flow")
            print_args.append(f"[cyan]{geo_alt}[/cyan]")

        table.add_row(*print_args)
        console.print(table)

    # If we're told to stop after one round, we tell the callback to exit by returning a False
    #
    if stop_after_one:
        return False
    else:
        return True

def _control_c_handler(sig, frame):
    sys.exit(0)

#
# Device monitor lets you watch traffic going across the device. If invoked by itself
# it runs interactively and shows each command until user hits Control-C.
# If --raw is specified, we display only the JSON retrieved.
# If --stop is specified, we stop after only showing a single incoming update. This and --raw can then
# be used for unit-testing end-to-end message transmission.
#
@device.command()
@common_cli_params
@click.option("--project", help="Project name", envvar="IOT_PROJECT", required=True)
@click.option("--model", help="Model name", envvar="IOT_MODEL", required=True)
@click.option("--serial", "--device", help="Device serial", required=True)
@click.option('--raw/--no-raw', help='Show raw data', default=False)
@click.option('--stop/--no-stop', help='Stop after receiving a single item', default=False)
def monitor(team, profile, project, model, serial, raw, stop):
    """
    Monitor device traffic for a given device. We use the device certs and topic
    to watch what is going on with a single device.
    \f
    Examples:

    $ iot device monitor --project "..." --serial "..."

    To stop the monitoring, hit Control-C to interrupt.
    """
    try:
        config = preload_config(team, profile)

        topic = f"simpleiot_v1/app/monitor/{project}/{model}/{serial}/#"
        subscribe_to_mqtt_topic(config.team, project, model, serial, topic, raw, stop, on_data_callback, on_error_callback)

        if not raw:
            print(f"-- Watching data for device [{serial}]")

        if not stop:
            print(f"-- To stop, press Control-C.\n")

        loop_count = 0
        signal.signal(signal.SIGINT, _control_c_handler)
        while True:
            loop_count += 1
            time.sleep(1)
    except Exception as e:
        print(f"ERROR: {str(e)}")




#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#

from setuptools import setup, find_packages
from pathlib import Path
import sys
import semver
import configparser
import os
import traceback

#
# This function automatically bumps up the current [version] section in setup.cfg file.
# By default it bumps up the last/patch version of the semantic version value (i.e. #.#.#)
# It then writes the value back out so each time you run pypideploy, the version is
# automatically pushed up.
#
# We've added a [version] section to the standard setup.cfg file. Two key/values there are:
# [version]
# pypi_test_version=1.0.1
# pypi_version=1.0.1
#
# Since this is usually loaded by the pypideploy script, it can set an environment variable
# indicating whether it's a test release or a production one. If not set, it will go out to
# the test one.
#
# That environment variable will indicate which version we will be bumping up.
#
def bump_and_return_version():
    initial_value = "1.0.0"
    bump_type = None
    result = None
    try:
        config = configparser.ConfigParser()
        config.read('setup.cfg')
        version_section = config["version"]
        if not version_section:
            config['version']['pypi_test_version'] = initial_value
            config['version']['pypi_version'] = initial_value
            result = initial_value
        else:
            is_full = os.environ.get("IOT_DEPLOY_PYPI", 0)
            if is_full == 1:
                bump_type = "PyPi"
                pypi_version_str = version_section.get('pypi_version')
                if pypi_version_str:
                    pypi_version = semver.VersionInfo.parse(pypi_version_str)
                    pypi_version = pypi_version.bump_patch()
                    version_section['pypi_version'] = str(pypi_version)
                    result = str(pypi_version)
            else:
                bump_type = "PyPi Test"
                pypi_test_version_str = version_section.get('pypi_test_version')
                if pypi_test_version_str:
                    pypi_test_version = semver.VersionInfo.parse(pypi_test_version_str)
                    pypi_test_version = pypi_test_version.bump_patch()
                    version_section['pypi_test_version'] = str(pypi_test_version)
                    result = str(pypi_test_version)

        with open('setup.cfg', 'w') as config_file:
            config.write(config_file)

        print(f"Bumping {bump_type} version to {result}")
        return result

    except Exception as e:
        print(f"ERROR processing setup.cfg file: {str(e)}")
        print(traceback.format_exc())
        exit(1)


with open(Path('requirements.txt')) as f:
    required = f.read().splitlines()

this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(name='simpleiot-cli',
      version=bump_and_return_version(),
      description='SimpleIOT command line interface',
      long_description=long_description,
      long_description_content_type='text/markdown',
      url='http://github.com/awslabs/simpleiot-cli',
      author='Ramin Firoozye',
      author_email='framin@amazon.com',
      license='Apache 2.0',
      py_modules=['iot'],
      packages=find_packages() + ["simpleiot/demo", "simpleiot/demo/m5gif"],
      package_data={'simpleiot/demo': ['*'],
                    'simpleiot/demo/m5gif': ['*']
                    },
      install_requires=required,
      download_url="https://github.com/awslabs/simpleiot-cli",
      keywords=["simpleiot", "simpleiot-cli", "iot", "aws", "awslabs", "cli"],
      classifiers=[
          "Environment :: Console",
          "Intended Audience :: Developers",
          "Intended Audience :: System Administrators",
          "Operating System :: Microsoft :: Windows",
          "Operating System :: MacOS :: MacOS X",
          "License :: OSI Approved :: Apache Software License",
          "Topic :: Software Development",
          "Topic :: Software Development :: Embedded Systems",
          "Topic :: System :: Hardware",
          "Topic :: System :: Installation/Setup",
          "Topic :: Utilities"
      ],
      entry_points={
          'console_scripts': [
              'iot=simpleiot.iot:iotcli'
          ],
      },
      )



#!/usr/bin/env python

# © 2022 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
#
# SimpleIOT project.
# Author: Ramin Firoozye (framin@amazon.com)
#
# This action allows user to authenticate against the Cognito/SSO userpool.
# Account credentials and the JWT token are stored in the local system secure pool.
# If username and password are specified, we overwrite any stored.
# They may also be specified in environment variables.
#
# The corresponding 'logout' action will log out and erase credentials.
#
import click
import json
from simpleiot.common.utils import *
from simpleiot.common.config import *

from rich import print
from rich.console import Console
import questionary
import boto3


console = Console()

# For this to work the Cognito user pool should have been set to ADMIN_NO_SRP_AUTH auth flow.
#
def _generate_token(config, username, password):
    access_token = None
    id_token = None
    cognito = None

    try:
        userpool = config.userpool
        client_id = config.client_id
        region = config.region

        if not userpool:
            print(f"INTERNAL ERROR: missing Cognito userpool in config file")
            exit(1)
        if not client_id:
            print(f"INTERNAL ERROR: missing Cognito Client ID in config file")
            exit(1)
        cognito = boto3.client('cognito-idp', region_name=region)
        resp = cognito.initiate_auth(
            ClientId=client_id,
            AuthFlow='USER_PASSWORD_AUTH',
            AuthParameters={
                "USERNAME": username,
                "PASSWORD": password
            }
        )

        access_token = resp['AuthenticationResult']['AccessToken']
        id_token = resp['AuthenticationResult']['IdToken']
    except cognito.exceptions.UserNotFoundException:
        print("ERROR: User not found")
    except cognito.exceptions.NotAuthorizedException:
        print("ERROR: login not authorized")
    except Exception as e:
        print(f"ERROR: {str(e)}")

    return access_token, id_token

@click.group()
def auth():
    """User Authentication"""
    pass

@auth.command()
@common_cli_params
@click.option("--username", "-user", help="Login username", envvar="IOT_USERNAME")
@click.option("--password", "-pass", help="Login password", envvar="IOT_PASSWORD")
@click.option("--clear", is_flag=True, help="Clear cached credentials")
def login(team, profile, username, password, clear):
    """
    Login to Team with auth credentials

    If a Team (specified in --team or via IOT_TEAM environment variable) specifies whether we've
    logged in using AWS SSO (Single Sign-on) we get credentials from the Keychain. If not specified
    we go through an SSO login process.

    If the team indicates that SimpleIOT was installed using standard Cognito login, this mechanism
    connects to Cognito and obtains a JWT token. The token is cached and used for subsequent connections
    to API gateway.

    If token is invalid, we'll quietly retry logging in using the cached username/password.
    These are stored in the local secure cache (i.e. Keychain for Mac).

    To get rid of all authentication tokens, add --clear to the command line.

    :return:
    """
    try:
        config = preload_config(team, profile)

        logged_in = False
        if clear:
            clear_all_auth(config)
            print("Done: Cached login credentials cleared")
            return

        if config.use_sso:
            # We assume if they're running this again, that they want fresh tokens obtained.
            #
            # access_key = get_stored_access_key(config)
            # access_secret = get_stored_access_secret(config)
            # session_token = get_stored_session_token(config)
            #
            # if access_key and access_secret and session_token:
            #     print("Logged in")
            # else:
            config_data = load_config(team)
            sso_url, sso_region, account_id, account_role, account_name, access_key, access_secret, session_token = \
                login_with_sso(config_data)
            if access_key:
                store_access_key(config, access_key)
            if access_secret:
                store_access_secret(config, access_secret)
            if session_token:
                store_session_token(config, session_token)
            print("Logged in")
        else:
            # If username or password is specified, we first check to see if we already have one in the
            # keyring. If so, these overwrite those (if different).
            # NOTE: stored username/password/taken routines are loaded from common.utils
            #
            stored_username = get_stored_username(config)
            stored_password = get_stored_password(config)

            if not username:
                username = stored_username

            while not username:
                username = questionary.text(f"Username:").ask()
                if not username:
                    print("Please enter username")

            if not password:
                password = stored_password

            while not password:
                password = questionary.password(f"Password:").ask()
                if not password:
                    print("Please enter password")

            # If password shows up with quotes on the outside, we try to strip it out.
            #
            password = unquote(password)
            access_token, id_token = _generate_token(config, username, password)
            if id_token:
                # print(f"Logged in with\nid_token: {id_token}\naccess_token: {access_token}")
                store_api_token(config, id_token)
                logged_in = True
                if username != stored_username:
                    store_username(config, username)
                if password != stored_password:
                    store_password(config, password)

            if logged_in:
                print(f"Logged in as user: {username}.")
            else:
                print(f"Failed to login.")
    except Exception as e:
        print(f"ERROR: {str(e)}")


@auth.command()
@common_cli_params
@click.option("--clear", is_flag=True, help="Clear cached credentials")
def logout(team, profile, clear):
    """
    Logout from Team and clear credentials
    """
    try:
        config = preload_config(team, profile)

        if clear:
            clear_all_auth(config)
            print("Done: All cached login credentials cleared")
        else:
            clear_api_token(config)
            print("Session logged out.")
    except Exception as e:
        print(f"ERROR: {str(e)}")


# @auth.command()
# @click.pass_obj
# @common_cli_params
# @click.option("--sso", prompt=True, help="SSO profile", envvar="IOT_SSO_PROFILE")
# def sso(team, profile, sso):
#     """
#     Login with Single-Sign-On
#     """
#     logged_in = False
#
#     try:
#         config = preload_config(team, profile)
#
#         region = config.region
#         if not profile:
#             profile = config.profile
#         if profile:
#             print(f"Logging in with SSO profile: {sso}")
#
#             boto3.setup_default_session(profile_name=sso, region_name=region)
#             sso_client = boto3.client('sso', region_name=region)
#             role_name = config.get("sso_role_name", "AdministratorAccess")
#             role_account = config.get("account", None)
#
#             # TODO: retrieve access token as described here:
#             # https://docs.aws.amazon.com/singlesignon/latest/OIDCAPIReference/API_CreateToken.html
#             #
#             access_token = None
#
#             if not role_account:
#                 print(f"ERROR: 'account' not configured in the config.json file for team")
#                 exit(1)
#
#             resp = sso_client.get_role_credentials(
#                 roleName=role_name, # Role (could be multiple)
#                 accountId=role_account, # sso_account_id,
#                 accessToken=access_token
#             )
#             access_id = resp['roleCredentials']['accessKeyId']
#             print(f"Got access ID: {access_id}")
#     except Exception as e:
#         print(f"ERROR: could not login using SSO: {str(e)}")
#         exit(1)



import sys
import io
import os

from PIL import Image
from PIL import GifImagePlugin


def gif2jpg(file_name: str, trans_color: tuple):
    """
    convert gif to `num_key_frames` images with jpg format
    :param file_name: gif file name
    :param num_key_frames: result images number
    :param trans_color: set converted transparent color in jpg image
    :return:
    """
    with Image.open(file_name) as im:
        num_key_frames = im.n_frames
        for i in range(num_key_frames):
            im.seek(im.n_frames // num_key_frames * i)
            image = im.convert("RGBA")
            datas = image.getdata()
            newData = []
            for item in datas:
                if item[3] == 0:  # if transparent
                    newData.append(trans_color)  # set transparent color in jpg
                else:
                    newData.append(tuple(item[:3]))
            image = Image.new("RGB", im.size)
            image.getdata()
            image.putdata(newData)
            filename = f"{i}.jpg"
            image.save(filename)


            # gif2jpg("image.gif", (255, 255, 255))  # convert image.gif to jpg images with white background

def write_frame_from_file(outfile, filename, frame_number):
    data_len = os.path.getsize(filename)
    
    with open(filename, 'rb') as infile:
        array_name = f"const unsigned char frame_{frame_number} [{data_len}] = {{" + "\n"
        outfile.write(array_name.encode('utf-8'))
        while True:
            data = infile.read(20)
            if len(data) > 0:
                # outfile.write('\t'.encode('utf-8'))
                for i in range(0, len(data)):
                    d = "0x%02x," % data[i]
                    outfile.write(d.encode('utf-8'))
                outfile.write('\n'.encode('utf-8'))
            else:
                outfile.write('};\n'.encode('utf-8'))
                break
                
    return data_len




def write_frame(outfile, frame, data):
    data_len = len(data)
    array_name = f"const unsigned char frame_{frame} [{data_len}] = {{" + "\n"
    outfile.write(array_name.encode('utf-8'))
    
    start = 0
    skip = 20
    
    count = data_len // skip
    remainder = data_len % skip
    
    for i in range(count):
        buffer = data[start:skip]
        buffer_len = len(buffer)
        for i in range(0, len(buffer)):
            d = "0x%02x," % data[i]
            outfile.write(d.encode('utf-8'))
        outfile.write('\n'.encode('utf-8'))

    if remainder > 0:
        start_remainder = count * skip
        buffer = data[start_remainder:]
        for i in range(0, len(buffer)):
            d = "0x%02x," % data[i]
            outfile.write(d.encode('utf-8'))
        outfile.write('\n'.encode('utf-8'))
        
    outfile.write('};\n\n'.encode('utf-8'))
    

def write_decl(output, frame_count,  image_width, image_height):
    decl_str = f"const unsigned int animated_frame_count = {frame_count};" + "\n"
    output.write(decl_str.encode('utf-8'))
    decl_str = f"const unsigned int animated_frame_width = {image_width};" + "\n"
    output.write(decl_str.encode('utf-8'))
    decl_str = f"const unsigned int animated_frame_height = {image_height};" + "\n"
    output.write(decl_str.encode('utf-8'))
    decl_str = f"const unsigned char* animated_frames[] = {{" + "\n"
    output.write(decl_str.encode('utf-8'))
    for i in range(frame_count):
        decl_str = f"   frame_{i}"
        if i < frame_count-1:
            decl_str += ","
        decl_str += "\n"
        output.write(decl_str.encode('utf-8'))
    decl_str = f"}};" + "\n\n"
    output.write(decl_str.encode('utf-8'))

    
def write_frame_size(output, image_data_size):
    decl_str = f"const unsigned int animated_frame_size[] = {{" + "\n"
    output.write(decl_str.encode('utf-8'))
    for size in image_data_size:
        decl_str = f"  {size}," + "\n"
        output.write(decl_str.encode('utf-8'))
    decl_str = f"}};" + "\n\n"
    output.write(decl_str.encode('utf-8'))
    

def write_header(output):
    decl_str = f"#include <M5Core2.h>" + "\n\n"
    output.write(decl_str.encode('utf-8'))
    

def gif2code(input, output, temp_file):
    with open(output, 'wb') as output:
        with Image.open(input) as im:
            num_key_frames = im.n_frames - 1
            image_width, image_height = im.size
            print(f"Frames: {num_key_frames}")
            frame_sizes = []
            
            for i in range(num_key_frames):
                seek_frame = im.n_frames // num_key_frames * i
                print(f"Getting frame: {seek_frame}")
                im.seek(seek_frame)
                image = im.convert("RGBA")
                datas = image.getdata()
                newData = []
                for item in datas:
                    if item[3] == 0:  # if transparent
                        newData.append(trans_color)  # set transparent color in jpg
                    else:
                        newData.append(tuple(item[:3]))
                image = Image.new("RGB", im.size)
                image.getdata()
                image.putdata(newData)
                image.save(temp_file)
        
#       with Image.open(input) as im:
#           # write_header(output)
#           num_key_frames = im.n_frames
#           image_width, image_height = im.size
#           
#           print(f"Frame: {num_key_frames}")
#           frame_sizes = []
#           
#           for i in range(num_key_frames):
#               im.seek(im.n_frames // num_key_frames * i)
#               image = im.convert("RGBA")
#               image_data = image.getdata()
#               new_data = []
#               for item in image_data:
#                   if item[3] == 0:  # see if transparent
#                       new_data.append(trans_color)
#                   else:
#                       new_data.append(tuple(item[:3]))
#               new_image = Image.new("RGB", im.size)
#               new_image.getdata()
#               new_image.putdata(new_data)
#               image.save(temp_file)

                frame_size = write_frame_from_file(output, temp_file, i)
                frame_sizes.append(frame_size)
                os.remove(temp_file)
                
#               with io.BytesIO() as memfile:
#                   new_image.save(memfile, "JPEG")
#                   frame_data = memfile.getvalue()
#                   frame_size = len(frame_data)
#                   write_frame(output, i, frame_data)
#                   frame_sizes.append(frame_size)

            write_frame_size(output, frame_sizes)
            write_decl(output, num_key_frames, image_width, image_height)
            

if __name__ == "__main__":
    source = "image.gif"
    output = "animation.c"
    temp_file = "_image_temp.jpg"
    
    if (len(sys.argv) > 1):
        source = sys.argv[1]
        if (len(sys.argv) > 2):
            output = sys.argv[2]

    gif2code(source, output, temp_file)
    print("Done!")

#   else:
#       print("Usage: gif2code [source animated gif file] [output C file]")
    